from django.contrib import admin
from django.utils.html import format_html
from imagekit import ImageSpec
from imagekit.admin import AdminThumbnail
from imagekit.cachefiles import ImageCacheFile
from imagekit.processors import ResizeToFit

from gallery.models import Album, City, Location, Photo, Redir


class AdminThumbnailSpec(ImageSpec):
    processors = [ResizeToFit(100, 100)]
    format = 'JPEG'
    options = {'quality': 60}


def cached_admin_thumb(instance):
    cached = ImageCacheFile(AdminThumbnailSpec(instance.photo))
    cached.generate()
    return cached


class RedirAdmin(admin.ModelAdmin):
    list_display = ('path', 'album', 'test_url')
    search_fields = ('path',)
    ordering = ('path',)
    list_per_page = 10
    list_editable = ('album',)

    def formfield_for_foreignkey(self, db_field, request, **kwargs):
        if db_field.name == "album":
            kwargs["queryset"] = Album.objects.all().order_by('name')  # Järjestä albumit aakkosjärjestykseen
        return super().formfield_for_foreignkey(db_field, request, **kwargs)

    def test_url(self, obj):
        return format_html(
            '<a href="/{}" target="_blank">http://nyymix.net/{}</a>',
            obj.path, obj.path
        )
    test_url.short_description = "Test redirection"


class CityAdmin(admin.ModelAdmin):
    list_display = ('name',)
    search_fields = ('name',)
    ordering = ('name',)
    list_per_page = 30


class LocationAdmin(admin.ModelAdmin):
    list_display = ('city', 'place')
    list_filter = ('city',)
    ordering = ('city__name',)
    search_fields = ('place', 'city__name',)
    list_per_page = 30


class AlbumAdmin(admin.ModelAdmin):
    prepopulated_fields = {'slug': ('name',)}
    list_display = ('__str__', 'name', 'location',  'album_date', 'is_public', 'thumbnail', )
    search_fields = ('name',)
    ordering = ('-album_date',)
    list_per_page = 20
    list_editable = ('name', 'is_public', 'location')
    readonly_fields = ['cover_preview']  # Lisätään esikatselukuva readonly_fieldsiin

    def cover_preview(self, obj):
        if obj.cover and obj.cover.photo:
            return format_html(
                '<img src="{}" style="max-width: 300px; height: auto;"/>',
                obj.cover.photo.url,
            )
        return "No cover image available"
    cover_preview.short_description = "Cover Preview"

    def formfield_for_foreignkey(self, db_field, request, **kwargs):
        if db_field.name == "cover":
            if hasattr(request, 'resolver_match') and request.resolver_match.kwargs.get('object_id'):
                album_id = request.resolver_match.kwargs['object_id']
                kwargs["queryset"] = Photo.objects.filter(album_id=album_id)

        return super().formfield_for_foreignkey(db_field, request, **kwargs)

    def thumbnail(self, obj):
        if obj.cover and obj.cover.photo:
            return format_html(
                '<img src="{}" style="width: 50px; height: auto;"/>',
                obj.cover.photo.url,
            )
        return "-"
    thumbnail.short_description = "Thumbnail"


class PhotoAdmin(admin.ModelAdmin):
    list_display = ('slug', 'album', 'is_favorite', 'admin_thumbnail',)
    list_display_links = ('slug',)
    search_fields = ('slug', 'photo',)
    readonly_fields = ['slug', 'taken_at', 'height', 'width', 'exif', ]
    admin_thumbnail = AdminThumbnail(image_field=cached_admin_thumb)
    list_filter = ('album',)
    list_per_page = 30
    list_editable = ('is_favorite',)


admin.site.register(City, CityAdmin)
admin.site.register(Location, LocationAdmin)
admin.site.register(Album, AlbumAdmin)
admin.site.register(Photo, PhotoAdmin)
admin.site.register(Redir, RedirAdmin)
